@@ -28,14 +28,14 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-commands.h,v 1.18 2006/08/07 16:42:40 pborelli Exp $
+ * $Id: gedit-commands.h,v 1.19 2006/08/16 17:45:04 pborelli Exp $
  */
 
 #ifndef __GEDIT_COMMANDS_H__
 #define __GEDIT_COMMANDS_H__
 
 #include <gtk/gtkaction.h>
-#include "gedit-window.h"
+#include <gedit/gedit-window.h>
 
 G_BEGIN_DECLS
 
