@@ -27,7 +27,7 @@
  * list of people on the gedit Team.  
  * See the ChangeLog files for a list of changes. 
  *
- * $Id: gedit-document.h,v 1.44 2006/01/05 18:35:10 paolo Exp $
+ * $Id: gedit-document.h,v 1.47 2006/08/07 18:04:07 pborelli Exp $
  */
  
 #ifndef __GEDIT_DOCUMENT_H__
@@ -118,7 +118,10 @@ typedef enum
 	GEDIT_DOCUMENT_SAVE_IGNORE_MTIME 	= 1 << 0,
 
 	/* write the file directly without attempting to backup */
-	GEDIT_DOCUMENT_SAVE_IGNORE_BACKUP	= 1 << 1
+	GEDIT_DOCUMENT_SAVE_IGNORE_BACKUP	= 1 << 1,
+	
+	/* preserve previous backup file, needed to support autosaving */
+	GEDIT_DOCUMENT_SAVE_PRESERVE_BACKUP	= 1 << 2
 } GeditDocumentSaveFlags;
 
 
@@ -151,12 +154,12 @@ gchar		*gedit_document_get_mime_type 	(GeditDocument       *doc);
 
 gboolean	 gedit_document_get_readonly 	(GeditDocument       *doc);
 
-gboolean	 gedit_document_load 		(GeditDocument       *doc, 
-						 const gchar         *uri, 
+void		 gedit_document_load 		(GeditDocument       *doc,
+						 const gchar         *uri,
 						 const GeditEncoding *encoding,
 						 gint                 line_pos,
 						 gboolean             create); 
-						 
+
 gboolean	 gedit_document_insert_file	(GeditDocument       *doc,
 						 GtkTextIter         *iter, 
 						 const gchar         *uri, 
@@ -219,19 +222,13 @@ GtkSourceLanguage
 const GeditEncoding 
 		*gedit_document_get_encoding	(GeditDocument       *doc);
 
-// CHECK: I think this can be private
-void		 gedit_document_set_auto_save_enabled	
-						(GeditDocument       *doc, 
-						 gboolean             enable);
-void		 gedit_document_set_auto_save_interval 
-						(GeditDocument       *doc, 
-						 gint                 interval);
-
 void		 gedit_document_set_enable_search_highlighting 
 						(GeditDocument       *doc,
 						 gboolean             enable);
+
 gboolean	 gedit_document_get_enable_search_highlighting
 						(GeditDocument       *doc);
+
 /* 
  * Non exported functions
  */
@@ -241,10 +238,6 @@ gboolean	_gedit_document_is_saving_as	(GeditDocument       *doc);
 glong		 _gedit_document_get_seconds_since_last_save_or_load 
 						(GeditDocument       *doc);
 
-/* private because the property will move to gtk */
-gboolean	 _gedit_document_get_has_selection
-						(GeditDocument       *doc);
-
 void		_gedit_document_search_region   (GeditDocument       *doc,
 						 const GtkTextIter   *start,
 						 const GtkTextIter   *end);
