@@ -47,7 +47,7 @@ gedit_prefs_manager_get_ ## name (void)					\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_get_bool (key,	\
+	return gedit_prefs_manager_get_bool (key,			\
 					     (def));			\
 }									\
 									\
@@ -56,7 +56,7 @@ gedit_prefs_manager_set_ ## name (gboolean v)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	gedit_prefs_manager_set_bool (key,		\
+	gedit_prefs_manager_set_bool (key,				\
 				      v);				\
 }									\
 				      					\
@@ -65,7 +65,7 @@ gedit_prefs_manager_ ## name ## _can_set (void)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_key_is_writable (key);\
+	return gedit_prefs_manager_key_is_writable (key);		\
 }	
 
 
@@ -75,7 +75,7 @@ gedit_prefs_manager_get_ ## name (void)			 		\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_get_int (key,		\
+	return gedit_prefs_manager_get_int (key,			\
 					    (def));			\
 }									\
 									\
@@ -84,7 +84,7 @@ gedit_prefs_manager_set_ ## name (gint v)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	gedit_prefs_manager_set_int (key,		\
+	gedit_prefs_manager_set_int (key,				\
 				     v);				\
 }									\
 				      					\
@@ -93,7 +93,7 @@ gedit_prefs_manager_ ## name ## _can_set (void)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_key_is_writable (key);\
+	return gedit_prefs_manager_key_is_writable (key);		\
 }		
 
 
@@ -103,7 +103,7 @@ gedit_prefs_manager_get_ ## name (void)			 		\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_get_string (key,	\
+	return gedit_prefs_manager_get_string (key,			\
 					       def);			\
 }									\
 									\
@@ -112,7 +112,7 @@ gedit_prefs_manager_set_ ## name (const gchar* v)			\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	gedit_prefs_manager_set_string (key,		\
+	gedit_prefs_manager_set_string (key,				\
 				        v);				\
 }									\
 				      					\
@@ -121,7 +121,7 @@ gedit_prefs_manager_ ## name ## _can_set (void)				\
 {									\
 	gedit_debug (DEBUG_PREFS);					\
 									\
-	return gedit_prefs_manager_key_is_writable (key);\
+	return gedit_prefs_manager_key_is_writable (key);		\
 }		
 
 
@@ -154,11 +154,6 @@ static gint		 gedit_prefs_manager_get_int		(const gchar* key,
 static gchar		*gedit_prefs_manager_get_string		(const gchar* key, 
 								 const gchar* def);
 
-gboolean		 gconf_client_set_color 		(GConfClient* client, 
-								 const gchar* key,
-								 GdkColor val, 
-								 GError** err);
-
 
 gboolean
 gedit_prefs_manager_init (void)
@@ -336,13 +331,15 @@ gconf_client_get_color_with_default (GConfClient* client, const gchar* key,
 	return color;
 }
 
-gboolean
-gconf_client_set_color (GConfClient* client, const gchar* key,
-                        GdkColor val, GError** err)
+static gboolean
+gedit_gconf_client_set_color (GConfClient   *client,
+                              const gchar   *key,
+                              GdkColor       val,
+                              GError       **err)
 {
 	gchar *str_color = NULL;
 	gboolean res;
-	
+
 	gedit_debug (DEBUG_PREFS);
 
 	g_return_val_if_fail (client != NULL, FALSE);
@@ -390,7 +387,7 @@ gedit_prefs_manager_set_color (const gchar* key, GdkColor value)
 	g_return_if_fail (gconf_client_key_is_writable (
 				gedit_prefs_manager->gconf_client, key, NULL));
 			
-	gconf_client_set_color (gedit_prefs_manager->gconf_client, key, value, NULL);
+	gedit_gconf_client_set_color (gedit_prefs_manager->gconf_client, key, value, NULL);
 }
 
 
@@ -402,13 +399,13 @@ DEFINE_BOOL_PREF (use_default_font,
 /* Editor font */
 DEFINE_STRING_PREF (editor_font,
 		    GPM_EDITOR_FONT,
-		    GPM_DEFAULT_EDITOR_FONT);
+		    GPM_DEFAULT_EDITOR_FONT)
 
 
 /* Use default colors */
 DEFINE_BOOL_PREF (use_default_colors,
 		  GPM_USE_DEFAULT_COLORS,
-		  GPM_DEFAULT_USE_DEFAULT_COLORS);
+		  GPM_DEFAULT_USE_DEFAULT_COLORS)
 
 
 /* Background color */	
@@ -523,7 +520,7 @@ gedit_prefs_manager_selection_color_can_set (void)
 /* Create backup copy */
 DEFINE_BOOL_PREF (create_backup_copy,
 		  GPM_CREATE_BACKUP_COPY,
-		  GPM_DEFAULT_CREATE_BACKUP_COPY);
+		  GPM_DEFAULT_CREATE_BACKUP_COPY)
 
 /* Backup extension. This is configurable only using gconftool or gconf-editor */
 gchar *
@@ -538,12 +535,12 @@ gedit_prefs_manager_get_backup_extension (void)
 /* Auto save */
 DEFINE_BOOL_PREF (auto_save,
 		  GPM_AUTO_SAVE,
-		  GPM_DEFAULT_AUTO_SAVE);
+		  GPM_DEFAULT_AUTO_SAVE)
 
 /* Auto save interval */
 DEFINE_INT_PREF (auto_save_interval,
 		 GPM_AUTO_SAVE_INTERVAL,
-		 GPM_DEFAULT_AUTO_SAVE_INTERVAL);
+		 GPM_DEFAULT_AUTO_SAVE_INTERVAL)
 
 
 /* Undo actions limit: if < 1 then no limits */
@@ -627,7 +624,7 @@ gedit_prefs_manager_wrap_mode_can_set (void)
 /* Tabs size */
 DEFINE_INT_PREF (tabs_size, 
 		 GPM_TABS_SIZE, 
-		 GPM_DEFAULT_TABS_SIZE);
+		 GPM_DEFAULT_TABS_SIZE)
 
 /* Insert spaces */
 DEFINE_BOOL_PREF (insert_spaces, 
@@ -647,7 +644,7 @@ DEFINE_BOOL_PREF (display_line_numbers,
 /* Toolbar visibility */
 DEFINE_BOOL_PREF (toolbar_visible,
 		  GPM_TOOLBAR_VISIBLE,
-		  GPM_DEFAULT_TOOLBAR_VISIBLE);
+		  GPM_DEFAULT_TOOLBAR_VISIBLE)
 
 
 /* Toolbar suttons style */
@@ -680,7 +677,6 @@ gedit_prefs_manager_get_toolbar_buttons_style (void)
 	g_free (str);
 
 	return res;
-
 }
 
 void
@@ -826,7 +822,7 @@ DEFINE_INT_PREF (print_line_numbers,
  * gnome_font_face_find_closest() (probably a gnome-print bug)
  */
 static void
-face_and_size_from_full_name (const guchar   *name,
+face_and_size_from_full_name (const gchar    *name,
 			      GnomeFontFace **face,
 			      gdouble        *size)
 {
@@ -843,7 +839,7 @@ face_and_size_from_full_name (const guchar   *name,
 		*size = 12;
 	}
 
-	*face = gnome_font_face_find_closest (copy);
+	*face = gnome_font_face_find_closest ((guchar *)copy);
 	g_free (copy);
 }
 
@@ -854,6 +850,7 @@ font_description_from_gnome_font_name (const char *font_name)
 	PangoFontDescription *desc;
 	PangoStyle style;
 	PangoWeight weight;
+	const gchar *family_name;
 	gdouble size;
 
 	face_and_size_from_full_name (font_name, &face, &size);
@@ -861,9 +858,10 @@ font_description_from_gnome_font_name (const char *font_name)
 	/* Pango and GnomePrint have basically the same numeric weight values */
 	weight = (PangoWeight) gnome_font_face_get_weight_code (face);
 	style = gnome_font_face_is_italic (face) ? PANGO_STYLE_ITALIC : PANGO_STYLE_NORMAL;
+	family_name = (const gchar *) gnome_font_face_get_family_name (face);
 
 	desc = pango_font_description_new ();
-	pango_font_description_set_family (desc, gnome_font_face_get_family_name (face));
+	pango_font_description_set_family (desc, family_name);
 	pango_font_description_set_weight (desc, weight);
 	pango_font_description_set_style (desc, style);
 	pango_font_description_set_size (desc, size * PANGO_SCALE);
@@ -1095,10 +1093,10 @@ gedit_prefs_manager_get_auto_detected_encodings (void)
 
 		      g_return_val_if_fail (charset != NULL, NULL);
 		      enc = gedit_encoding_get_from_charset (charset);
-		      
+
 		      if (enc != NULL)
 		      {
-			      if (!data_exists (res, (const gpointer)enc))
+			      if (!data_exists (res, (gpointer)enc))
 				      res = g_slist_prepend (res, (gpointer)enc);
 
 		      }
@@ -1145,13 +1143,13 @@ gedit_prefs_manager_get_shown_in_menu_encodings (void)
 
 		      if (strcmp (charset, "CURRENT") == 0)
 			      g_get_charset (&charset);
-      
+
 		      g_return_val_if_fail (charset != NULL, NULL);
 		      enc = gedit_encoding_get_from_charset (charset);
-		      
+
 		      if (enc != NULL)
 		      {
-			      if (!data_exists (res, (const gpointer)enc))
+			      if (!data_exists (res, (gpointer)enc))
 				      res = g_slist_prepend (res, (gpointer)enc);
 		      }
 
@@ -1289,6 +1287,27 @@ gedit_prefs_manager_get_restore_cursor_position (void)
 					     GPM_DEFAULT_RESTORE_CURSOR_POSITION);
 }
 
+/* Global Lockdown */
+GeditLockdownMask
+gedit_prefs_manager_get_lockdown (void)
+{
+	guint lockdown = 0;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_COMMAND_LINE, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_COMMAND_LINE;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_PRINTING, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_PRINTING;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_PRINT_SETUP, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_PRINT_SETUP;
+
+	if (gedit_prefs_manager_get_bool (GPM_LOCKDOWN_SAVE_TO_DISK, FALSE))
+		lockdown |= GEDIT_LOCKDOWN_SAVE_TO_DISK;
+
+	return lockdown;
+}
+
 /* The following functions are taken from gconf-client.c 
  * and partially modified. 
  * The licensing terms on these is: 
